@@ -7,12 +7,15 @@
   #' sparsity is allowed using the strategy options ("sparselv" or "kplusone") and the rho parameter. 
   
   #' @param X : The matrix of the predictors, the variables to be clustered
-  #' @param y : The response variable (numeric)
+  #' @param y : The response variable (usually numeric)
+  #'            If y is binary factor, indicator variable (0/1) is generated. A Bayes rule is used to compute class probabilities.\cr
+  #'            Performance criteria is RMSE for numerical variable; RMSE and error rate for binary factor. \cr
   #' @param method : The criterion to be use in the cluster analysis.  \cr
   #'        1 or "directional" : the squared covariance is used as a measure of proximity (directional groups). \cr    
   #'        2 or "local"       : the covariance is used as a measure of proximity (local groups)
   #' @param sX : TRUE/FALSE, i.e. standardization or not of the columns X (TRUE by default)
-  #' @param shrinkp : shrinkage  paramater used in the boosting (max : 1, 0.5 by default)
+  #' @param shrinkp : shrinkage  paramater used in the boosting (max : 1, 0.5 by default). \cr
+  #'         If shrinkp is a vector of positive values greater than 0, and lower or equal to 1, the outputs are given for each value.
   #' @param strategy : "none" (by default), or "kplusone" (an additional cluster for the unclassifiable variables),
   #'        or "sparselv" (zero loadings for the unclassifiable variables)
   #' @param rho : a threshold of correlation between 0 and 1 (used in "kplusone" or "sparselv" strategy, 0.3 by default)
@@ -26,36 +29,49 @@
   #' @return \item{Load}{ a list for the loadings of the X variables in the latent component.}
   #' @return \item{Alpha}{ a list of the regression coefficients to be applied to the latent components. \cr
   #'         The coefficients are aggregated when the same latent component is extracted several times during the iterative steps.}
-  #' @return \item{Beta}{ a list of the beta coefficients to be applied to the scaled predictors. \cr
+  #' @return \item{Beta}{ a list of the beta coefficients to be applied to the pretreated predictors. \cr
   #'         For a model with the A first latent components, the A first elements of the list must be added together.}  
   #' @return \item{GroupImp}{ Group Importance i.e. the decrease of the residuals' variance provided by the CLV components in the model.}
-  #' @return \item{RMSE.cal}{ the root mean square error for each observation of the data/training set, at each step of the procedure.}
+  #' @return \item{RMSE.cal}{ the root mean square error for the calibration set, at each step of the procedure.}
+  #' @return \item{ERRrate.cal and rocAUC.cal}{ when y is a binary factor, the classification rate and the AUC for ROC, on the bassis of the calibration set, at each step of the procedure.}
   #' @return \item{RMSE.val}{ as RMSE.cal but for the test set, if provided.}
-  #'
+  #' @return \item{ERRrate.val and rocAUC.val}{as for calibration set but for the test set, if provided.}
+  #' 
   #' @seealso CLV, CLV_kmeans
   #'
   #' @export
   #' 
 lm_CLV = function(X,y,method="directional",sX=TRUE,shrinkp=0.5, strategy="none",rho=0.3, validation=FALSE,id.test=NULL,maxiter=100, threshold=10e-6 )
 {  
- 
+  
   if(is.null(method)) stop('parameter method should be =1/"directional" or =2/"local"')
   if (method=="directional") method=1
   if (method=="local") method=2
   
-  Group = list()
-  Load=list()
-  Comp =list()
-  Beta=list() #matrix(0,maxiter,ncol(X))
-  Alpha=list()
   n = dim(X)[1]
   p = dim(X)[2]
   gpmax=p
   #maxiter=maxiter/shrinkp
-
+  
+  yisfact=FALSE
+  yfact=NULL
+  ERRrate.cal=NULL
+  ERRrate.val=NULL
+  rocAUC.cal=NULL
+  rocAUC.val=NULL
+  if (is.factor(y)) {
+    yfact=y
+    y=as.numeric(yfact)-1
+    yisfact=TRUE
+    n1=sum(y==1)
+    n0=sum(y==0)
+  }
   
   if (!validation) {
     Xscale=scale(X,scale=sX)
+    scale.mean<-apply(X,2,mean)
+    scale.scale<-apply(X,2,sd)
+    meany=mean(y)
     ycenter=scale(y,scale=FALSE)
   }
   if (validation) {
@@ -88,19 +104,28 @@ lm_CLV = function(X,y,method="directional",sX=TRUE,shrinkp=0.5, strategy="none",
   #end.time <- Sys.time()
   #time.taken <- end.time - start.time;          print(time.taken)
   
+# ==============================================================  
+# for each value of the shrinkage parameter 
+  resul=list()
+  for (valnu in 1:length(shrinkp))  {
+    Group = list()
+    Load=list()
+    Comp =list()
+    Beta=list() #matrix(0,maxiter,ncol(X))
+    Alpha=list()
  
  # ------------------------------------------------------------
  # iterative search for groups of X variables (and associated LV) well correlated with y
-  #yt=rep(mean(y),length(y))
+                        # yt=rep(mean(y),length(y))
   et=ycenter
   iter=0
-  #stockyt=yt
+                         # stockyt=yt
   stocket=et
   Xscale=scale(X,scale=sX)
  
   while (iter<maxiter) {
       iter=iter+1
-                               print(iter)
+                               #print(iter)
       tabgpy=matrix(0,p,gpmax)
       idgpy=rep(0,gpmax)
       compgpy=matrix(0,n,gpmax)
@@ -132,7 +157,7 @@ lm_CLV = function(X,y,method="directional",sX=TRUE,shrinkp=0.5, strategy="none",
   
 
       # ------------------------------------------------------------------------
-      # updatin of groups and latent component if strategy <>"none
+      # updating of groups and latent components if strategy <>"none
       # only the group "gpy" at each level of "nivparti", and only one iteration (for simplicity)
       # ------------------------------------------------------------------------
       if (strategy=="sparselv"){
@@ -175,19 +200,24 @@ lm_CLV = function(X,y,method="directional",sX=TRUE,shrinkp=0.5, strategy="none",
       }
       #tabgpyniv=reschoice$tabgpyniv
       rescrit=as.data.frame(rescrit) 
-      nf=rescrit[which(rescrit$KGX==1)[1],1];
+      change=which(diff(rescrit$KGX)==1)
+      change=change[length(change)]+1     # with this option, selection of the first step the groups remain unidimensional
+      nf=rescrit[change,1];
+      if (length(nf)==0) {nf=nivparti[length(nivparti)]}   # if no solution nf found, choice of the last solution
 
       # ------------------------------------------------------------------------
       # extraction of the Group and Comp, Evaluation of Load at this iteration/step
       # ------------------------------------------------------------------------
-      if (!is.na(nf)) {
+      #if (!is.na(nf)) {
           ind=memorelate[[which(nivparti==nf)]]
           indname=colnames(X)[ind]
           group=indname
           Xiter=as.matrix(Xscale[,ind])
           if (method==1) {
-            peig=powerEigen(crossprod(Xiter))
-            load=as.matrix(peig$vectors)
+            #peig=powerEigen(crossprod(Xiter))
+            #load=as.matrix(peig$vectors)
+            reseig=eigen(crossprod(Xiter))
+            load=as.matrix(reseig$vectors[,1])
             sig=sign(load[which.max(load)])
             load=load*sig
           }
@@ -195,32 +225,33 @@ lm_CLV = function(X,y,method="directional",sX=TRUE,shrinkp=0.5, strategy="none",
             load=matrix(1/length(ind),length(ind),1)
           }
           rownames(load)=colnames(Xiter)
-          comp=as.matrix(memocomp[,which(nivparti==nf)])     # idem :Xiter%*%load
+          comp=Xiter%*%load      # idem...but the sign as : as.matrix(memocomp[,which(nivparti==nf)]) 
           #res=extraction(nivparti,tabgpyniv,nf,res.clvX,strategy,Xscale)
           Group[[iter]]=group
           Load[[iter]]=load
           Comp[[iter]]=comp
-      } else {
-          break
-      }
+      # } else {
+      #     break
+      # }
       
       
       # ---------------------------------------------------------------
       # boosted-like step : beta, prediction and deflation
       # --------------------------------------------------------------
-        alpha=shrinkp*(cov(et,comp)/var(comp)) 
-        Alpha[[iter]]=c(alpha)
-        # ouind=apply(sapply(Group,FUN="==",colnames(X)),1,sum)
-        # ind=which(ouind==1)
-        Beta[[iter]]=rep(0,ncol(X))
-        Beta[[iter]][ind]=as.numeric(alpha)*load
-        yiter=comp%*%alpha
-        et= as.vector( et - yiter )   
-        stocket=cbind(stocket,et);
-        #yt=yt+yiter
-        #stockyt=cbind(stockyt,yt)
-        #memo=memoiter[1:which(nivparti==nf),]
-                                     
+          alpha=shrinkp[valnu]*(cov(et,comp)/var(comp)) 
+          Alpha[[iter]]=c(alpha)
+          # ouind=apply(sapply(Group,FUN="==",colnames(X)),1,sum)
+          # ind=which(ouind==1)
+          Beta[[iter]]=rep(0,ncol(X))
+          Beta[[iter]][ind]=as.numeric(alpha)*load
+          yiter=comp%*%alpha
+          et= as.vector( et - yiter )   
+          stocket=cbind(stocket,et);
+                                # yt=yt+yiter
+                                # stockyt=cbind(stockyt,yt)
+                                # print(stockyt)
+                                #memo=memoiter[1:which(nivparti==nf),]
+       
         # stopping rule                                         
         # if relative variance of et < threshold (10e-6 by default)
         critere=(var(stocket[,iter])-var(stocket[,iter+1]))  /var(y)
@@ -269,24 +300,47 @@ lm_CLV = function(X,y,method="directional",sX=TRUE,shrinkp=0.5, strategy="none",
      }
    }
     GroupImp=apply(resdetail,2,sum)
- 
+    # re_order the group according to their importance
+    ordr=order(GroupImp,decreasing=TRUE)
+    GroupImp=GroupImp[ordr]
+    uGroup <- uGroup[ordr]
+    uLoad <- uLoad[ordr]
+    uComp <- uComp[ordr]
+    uAlpha <- uAlpha[ordr]
+    uBeta <- uBeta[ordr]
     
   # ##########################################
   # final section : prediction
   RMSE.cal=NULL
+  if (yisfact) { ERRrate.cal=NULL; rocAUC.cal=NULL} 
   # RMSP.cal=NULL  
   if (validation)    {
     RMSE.val=NULL
+    if (yisfact) { ERRrate.val=NULL; rocAUC.val=NULL} 
   # RMSP.val=NULL
     Xstest<-scale(Xtest,center=scale.mean,scale=scale.scale)
   }
   # step "cst model"
   ypred=rep(mean(y),length(y))
-  RMSE.cal=c(sqrt(mean((y-ypred)^2)))
+  RMSE.cal=sqrt(mean((y-ypred)^2))
+  if (yisfact) {
+    m1=mean(ypred[y==1]);  s1=sd(ypred[y==1]);
+    m0=mean(ypred[y==0]);  s0=sd(ypred[y==0]);
+    tab=table(y,Bayes_classif(ypred,n1,m1,s1,n0,m0,s0)$yclass)
+    ERRrate.cal=(length(y)-sum(diag(tab)))/length(y)
+    W=wilcox.test(Bayes_classif(ypred,n1,m1,s1,n0,m0,s0)$pclass~ y,exact=FALSE)$statistic
+    rocAUC.cal=1-(W/(sum(y==1)*sum(y==0)))
+  }
   # RMSP.cal=c(sqrt(mean(scale(ypred,scale=FALSE)^2)))
   if (validation) {
     ypredtest=rep(meany,length(ytest))
     RMSE.val=c(sqrt(mean((ytest-ypredtest)^2)))
+    if (yisfact) {
+      tab=table(ytest,Bayes_classif(ypredtest,n1,m1,s1,n0,m0,s0)$yclass)
+      ERRrate.val=(length(ytest)-sum(diag(tab)))/length(ytest)
+      W=wilcox.test(Bayes_classif(ypredtest,n1,m1,s1,n0,m0,s0)$pclass~ ytest,exact=FALSE)$statistic
+      rocAUC.val=1-(W/(sum(ytest==1)*sum(ytest==0)))
+    }
   # RMSP.val=c(sqrt(mean(scale(ypredtest,scale=FALSE)^2)))
   }
   # step by step
@@ -294,21 +348,38 @@ lm_CLV = function(X,y,method="directional",sX=TRUE,shrinkp=0.5, strategy="none",
      ypred=ypred+Xscale%*%Beta[[iter]]
      #ERR.cal=cbind(ERR.cal,y-ypred)
      RMSE.cal=c(RMSE.cal,sqrt(mean((y-ypred)^2)))
+     if (yisfact) {
+       m1=mean(ypred[y==1]);  s1=sd(ypred[y==1]);
+       m0=mean(ypred[y==0]);  s0=sd(ypred[y==0]);
+       tab=table(y,Bayes_classif(ypred,n1,m1,s1,n0,m0,s0)$yclass)
+       ERRrate.cal=c(ERRrate.cal,(length(y)-sum(diag(tab)))/length(y))
+       W=wilcox.test(Bayes_classif(ypred,n1,m1,s1,n0,m0,s0)$pclass~ y,exact=FALSE)$statistic
+       rocAUC.cal=c(rocAUC.cal,1-(W/(sum(y==1)*sum(y==0))))
+     }
      #RMSP.cal=c(RMSP.cal,sqrt(mean(scale(ypred,scale=FALSE)^2)))
      if (validation){
           ypredtest=ypredtest+Xstest%*%Beta[[iter]]
           #ERR.val=cbind(ERR.val,ytest-ypredtest)
           RMSE.val=c(RMSE.val,sqrt(mean((ytest-ypredtest)^2)))
+          if (yisfact) {
+            tab=table(ytest,Bayes_classif(ypredtest,n1,m1,s1,n0,m0,s0)$yclass)
+            ERRrate.val=c(ERRrate.val,(length(ytest)-sum(diag(tab)))/length(ytest))
+            W=wilcox.test(Bayes_classif(ypredtest,n1,m1,s1,n0,m0,s0)$pclass~ ytest,exact=FALSE)$statistic
+            rocAUC.val=c(rocAUC.val,1-(W/(sum(ytest==1)*sum(ytest==0))))
+          }
           #RMSP.val=c(RMSP.val,sqrt(mean(scale(ypredtest,scale=FALSE)^2)))
      } 
    }
                   
-                   
+    if (!validation)  resul[[valnu]]=list(shrinkp=shrinkp[valnu],Group=uGroup,Load=uLoad,Comp=uComp,Alpha=uAlpha,Cst=meany,Beta=uBeta,GroupImp=GroupImp,RMSE.cal=RMSE.cal,ERRrate.cal=ERRrate.cal,rocAUC.cal=rocAUC.cal,resdetail=resdetail,sX=sX,X=X,yfact=yfact)
+    if (validation)   resul[[valnu]]=list(shrinkp=shrinkp[valnu],Group=uGroup,Load=uLoad,Comp=uComp,Alpha=uAlpha,Cst=meany,Beta=uBeta,GroupImp=GroupImp,RMSE.cal=RMSE.cal,ERRrate.cal=ERRrate.cal,rocAUC.cal=rocAUC.cal,RMSE.val=RMSE.val,ERRrate.val=ERRrate.val,rocAUC.val=rocAUC.val, resdetail=resdetail,sX=sX,X=X,yfact=yfact)
+ 
+  }
+  # end of for the valnu values
+  #=======================================================================
   
-    if (!validation)  res=list(Group=uGroup,Load=uLoad,Comp=uComp,Alpha=uAlpha,Beta=uBeta,GroupImp=GroupImp,RMSE.cal=RMSE.cal,resdetail=resdetail)
-    if (validation)   res=list(Group=uGroup,Load=uLoad,Comp=uComp,Alpha=uAlpha,Beta=uBeta,GroupImp=GroupImp,RMSE.cal=RMSE.cal,RMSE.val=RMSE.val,resdetail=resdetail)
-  
-    return(res)
+  class(resul) = "lmclv"
+  return(resul)
 }
 
 
